@@ -609,6 +609,7 @@ public class DFABuilder implements NinaAction {
 	private static final Object _FLOAT = new Object();
 	private static final Object _ID_JAVA = new Object();
 	private static final Object _ID_C = new Object();
+	private static final Object _STRING_LITERAL = new Object();
 
 	//
 	private static final Range SPACES = CharSets.parse(" \t\r\n\b");
@@ -1595,6 +1596,68 @@ public class DFABuilder implements NinaAction {
 		}
 	}
 
+	//
+	private void _linkStringLiteral(NinaEvent q, DBS v,
+			int enclosing) {
+		NinaLanguageOption p;
+		DBS x, esc, un1, un2, un3, un4, oc1, oc2, ded;
+		String unicodeBuf;
+
+		p = q.getLanguageOption();
+		x = createState(q);
+		_linkv(q, v, x, enclosing,
+				p.getInitBuffer(p.getSequenceBufferName()));
+		_linkv(q, x, vertex, enclosing, null);
+		x.linkOthers(x, p.getAppendBuffer(p.getSequenceBufferName()));
+
+		esc = createState(q);
+		_linkv(q, x, esc, '\\', null);
+		_linkv(q, esc, x, 'b', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\b'));
+		_linkv(q, esc, x, 't', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\t'));
+		_linkv(q, esc, x, 'n', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\n'));
+		_linkv(q, esc, x, 'f', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\f'));
+		_linkv(q, esc, x, 'r', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\r'));
+		_linkv(q, esc, x, '\"', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\"'));
+		_linkv(q, esc, x, '\'', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\''));
+		_linkv(q, esc, x, '\\', p.getAppendBuffer(
+				p.getSequenceBufferName(), '\\'));
+
+		unicodeBuf = p.getSequenceBufferName("Unicode");
+		un1 = createState(q);
+		un2 = createState(q);
+		un3 = createState(q);
+		un4 = createState(q);
+		_linkv(q, esc, un1, 'u', p.getInitBuffer(unicodeBuf));
+		un1.linkAlphabet(un2, HEXD, p.getAppendBuffer(unicodeBuf));
+		un2.linkAlphabet(un3, HEXD, p.getAppendBuffer(unicodeBuf));
+		un3.linkAlphabet(un4, HEXD, p.getAppendBuffer(unicodeBuf));
+		un4.linkAlphabet(x,   HEXD,
+				p.getAppendBuffer(unicodeBuf) +
+				p.getAppendBuffer(
+						p.getSequenceBufferName(), unicodeBuf, 16));
+
+		oc1 = createState(q);
+		oc2 = createState(q);
+		esc.linkAlphabet(oc1, OCTD,
+				p.getInitBuffer(unicodeBuf) +
+				p.getAppendBuffer(unicodeBuf));
+		oc1.linkAlphabet(oc2, OCTD, p.getAppendBuffer(unicodeBuf));
+		oc2.linkAlphabet(x,   OCTD,
+				p.getAppendBuffer(unicodeBuf) +
+				p.getAppendBuffer(
+						p.getSequenceBufferName(), unicodeBuf, 8));
+
+		ded = createState(q);
+		_linkv(q, x, ded, '\n', null);
+	}
+
 	/* (non-Javadoc)
 	 * @see net.morilib.nina.NinaAction#link(net.morilib.nina.NinaEvent, java.lang.Object)
 	 */
@@ -1665,6 +1728,8 @@ public class DFABuilder implements NinaAction {
 			_linkId(q, v, ID_JAVA_START, ID_JAVA_PART);
 		} else if(o == _ID_C) {
 			_linkId(q, v, ID_C_START, ID_C_PART);
+		} else if(o == _STRING_LITERAL) {
+			_linkStringLiteral(q, v, '\"');
 		} else {
 			v.linkAlphabet(vertex, o, mealy);
 		}
@@ -1996,8 +2061,12 @@ public class DFABuilder implements NinaAction {
 	public void setEdgeIdentifier(NinaEvent q, String lang) {
 		if(lang.equals("Java")) {
 			q.setEdge(_ID_JAVA);
-		} else {
+		} else if(lang.equals("C")) {
 			q.setEdge(_ID_C);
+		} else if(lang.equals("String")) {
+			q.setEdge(_STRING_LITERAL);
+		} else {
+			throw new RuntimeException();
 		}
 	}
 
