@@ -23,7 +23,7 @@ package net.morilib.nina;
 public class NinaNondeterministicException extends NinaException {
 
 	public NinaNondeterministicException(String s) {
-		super("nondeteministic", s);
+		super("nondeterministic", s);
 	}
 
 }
