@@ -1,3 +1,4 @@
+#! /bin/jsh
 #
 # Copyright 2013-2014 Yuichiro Moriguchi
 #
@@ -26,6 +27,8 @@ then
   nullsym='-1'
   [ -z "$NEWLINEMODE" ] && streamsym='java.io.Reader'
   [ -z "$NEWLINEMODE" ] || streamsym='java.io.InputStream'
+  streamfile='java.io.FileInputStream'
+  [ -z "$NEWLINEMODE" ] && streamins='java.io.InputStreamReader'
   tre='throws java.io.IOException'
 elif [ -z "$LTYPE" ]
 then
@@ -33,6 +36,8 @@ then
   nullsyml='-1'
   [ -z "$NEWLINEMODE" ] && streamsym='java.io.Reader'
   [ -z "$NEWLINEMODE" ] || streamsym='java.io.InputStream'
+  streamfile='java.io.FileInputStream'
+  [ -z "$NEWLINEMODE" ] && streamins='java.io.InputStreamReader'
   tre='throws java.io.IOException'
 else
   nullsym='null'
@@ -40,10 +45,15 @@ else
   streamsym='java.util.Iterator'
   tre=''
 fi
+
 buffer='$buffer'
+bufferUnicode='$bufferUnicode'
 int='$int'
 bigint='$bigint'
 num='$num'
+indent='$indent'
+indentBefore='$indentBefore'
+sq="'"
 
 echo '/**'
 awk '{ printf(" * %s\n", $0) }' description
@@ -51,12 +61,15 @@ echo ' */'
 cat << EOF
 public $ABSTRACT class ${CLASSNAME}${TEMPLATE} $extends $implements {
 
+	/* @@@-PARSER-CODE-START-@@@ */
 	static class TokenException extends RuntimeException {
 	}
 
 	static abstract class Engine {
-		abstract int step($streamsym __rd, $CTYPE c) $tre;
+		abstract int step($CTYPE c) $tre;
 		abstract boolean accepted();
+		abstract boolean isDead();
+		abstract boolean isEmptyTransition();
 		abstract int execaction($CTYPE c);
 		abstract boolean isend();
 		abstract int recover(Exception e);
@@ -104,6 +117,7 @@ cat << EOF
 	private static final int NINA_STACKLEN = 72;
 	static final int NINA_DISCARDSTATE = 0x40000000;
 	static final int INITIAL = 0;
+	static final int INDENT = 1;
 
 	private int STATE;
 	private int[] __sts = new int[NINA_STACKLEN];
@@ -112,17 +126,20 @@ cat << EOF
 	private int __slen = 0;
 	private $CTYPE unread = $nullsym;
 
-	$VALTYPE _;
+	$VALTYPE _v;
 	java.util.List<$VALTYPE> _l;
 	Object yieldObject;
-	$streamsym befstream = null;
 	Throwable exception;
 
 	StringBuffer $buffer;
+	StringBuffer $bufferUnicode;
 	int $int;
 	java.math.BigInteger $bigint;
 	Number $num;
 
+	java.util.Stack<$streamsym> streamStack =
+			new java.util.Stack<$streamsym>();
+
 	void _initlist() {
 		_l = new java.util.ArrayList<$VALTYPE>();
 	}
@@ -133,6 +150,14 @@ cat << EOF
 
 EOF
 
+[ -n "$ENABLE_INDENT" ] && cat << EOF
+	private int $indent;
+	private int $indentBefore;
+	private boolean __is_line_begin = true;
+	private int __indent_unread = -1;
+
+EOF
+
 [ -n "$ENABLE_LOG" ] && cat << EOF
 	private static final String __LOGFILE =
 			System.getProperty("nina.${CLASSNAME}.log.file");
@@ -144,6 +169,11 @@ EOF
 EOF
 [ -z "$TYPE" ] && [ -n "$ENABLE_MATCHER" ] && echo '	private String matched;'
 
+if [ -n "$TYPE" ]; then
+elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
+  echo '	private int __unreadcomment = -2;'
+fi
+
 if [ -n "$ENABLE_BACKTRACK" ]
 then
   echo "	private $CTYPE[] __backtrack = null;"
@@ -172,45 +202,224 @@ fi
 
 echo
 cat field
-[ -n "$LEXER" ] && echo "private $LCTYPE _unreadl = $nullsyml;"
+[ -n "$LEXER" ] && echo "	private $LCTYPE _unreadl = $nullsyml;"
+[ -n "$LEXER" ] || echo "	private $CTYPE _unreadl = $nullsym;"
 
 function _putreadf {
-  echo "	$2 $1($streamsym stream) $tre {"
+  echo
+  echo "	void INCLUDE($streamsym rd) {"
+  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		if(__lookahead_ptr >= 0) {'
+  [ -n "$ENABLE_LOOKAHEAD" ] && echo '			throw new IllegalStateException();'
+  [ -n "$ENABLE_LOOKAHEAD" ] && echo '		}'
+  [ -n "$ENABLE_BACKTRACK" ] && echo '		if(__backtrack_ptr >= 0) {'
+  [ -n "$ENABLE_BACKTRACK" ] && echo '			throw new IllegalStateException();'
+  [ -n "$ENABLE_BACKTRACK" ] && echo '		}'
+  [ -n "$ENABLE_INDENT"    ] && echo '		if(__indent_unread >= 0) {'
+  [ -n "$ENABLE_INDENT"    ] && echo '			throw new IllegalStateException();'
+  [ -n "$ENABLE_INDENT"    ] && echo '		}'
+  echo "		streamStack.push(rd);"
+  echo "	}"
+  echo
+  if [ -n "$streamfile" ]; then
+    echo "	void INCLUDE(String name) $tre {"
+    echo '		java.io.InputStream ins;'
+    echo
+    echo "		ins = new ${streamfile}(name);"
+    [ -n "$streamins" ] && echo "		INCLUDE(new ${streamins}(ins));"
+    [ -n "$streamins" ] || echo "		INCLUDE(ins);"
+    echo '	}'
+    echo
+  fi
+
+  if [ -n "$3" ]; then
+  elif [ "$COMMENT" = "shell" ]; then
+    cat << EOF
+	int readComment() $tre {
+		int c;
+
+		c = streamStack.peek().read();
+		if(c == ${sq}${COMMENTHEADER}${sq}) {
+			while((c = streamStack.peek().read()) >= 0) {
+				if(c == '\n') {
+					break;
+				}
+			}
+			return c;
+		} else {
+			return c;
+		}
+	}
+EOF
+  elif [ "$COMMENT" = "Java" -o "$COMMENT" = "C" ]; then
+    cat << EOF
+	int readComment() $tre {
+		int c, state = 0;
+
+		if(__unreadcomment >= -1) {
+			c = __unreadcomment;
+			__unreadcomment = -2;
+			return c;
+		} else if((c = streamStack.peek().read()) == '/') {
+			while((c = streamStack.peek().read()) >= 0) {
+				switch(state) {
+				case 0:
+					if(c == '*') {
+						state = 100;
+					} else if(c == '/') {
+						state = 200;
+					} else {
+						__unreadcomment = c;
+						return '/';
+					}
+					break;
+				case 100:
+					if(c == '*') {
+						state = 101;
+					}
+					break;
+				case 101:
+					if(c == '/') {
+						return ' ';
+					} else {
+						state = 100;
+					}
+					break;
+				case 200:
+					if(c == '\n') {
+						return '\n';
+					}
+					break;
+				}
+			}
+			__unreadcomment = -1;
+			return '/';
+		} else {
+			return c;
+		}
+	}
+EOF
+  else
+    cat << EOF
+	int readComment() $tre {
+		return streamStack.peek().read();
+	}
+EOF
+  fi
+
+  [ -n "$ENABLE_INDENT" ] || echo "	$2 $1() $tre {"
+  [ -n "$ENABLE_INDENT" ] && echo "	$2 ${1}_1() $tre {"
   if [ -s read ]; then
     cat read
   elif [ -z "$3" ]; then
     cat << EOF
 		int c;
 
-		c = stream.read();
-		return c < 0 ? NINA_EOF : c;
+		while(streamStack.size() > 0) {
+			if((c = readComment()) >= 0) {
+				return c;
+EOF
+    [ -n "$streamfile" ] && echo '			} else if(streamStack.size() > 1) {'
+    [ -n "$streamfile" ] && echo '				streamStack.pop().close();'
+    cat << EOF
+			} else {
+				streamStack.pop();
+			}
+		}
+		return NINA_EOF;
 EOF
   else
     echo "		return ($2)stream.next();"
   fi
   echo '	}'
+
+  if [ -n "$ENABLE_INDENT" ]; then
+    cat << EOF
+	$2 $1() $tre {
+		int c;
+
+		if(__indent_unread >= 0) {
+			c = __indent_unread;
+			__indent_unread = -1;
+		} else if(!__is_line_begin) {
+			c = ${1}_1();
+		} else {
+			$indent = 0;
+			while((c = ${1}_1()) == ' ' || c == '\t') {
+				$indent++;
+			}
+			__indent_unread = c;
+			c = c >= 0 ? INDENT : c;
+		}
+
+		if(__is_line_begin = c == '\n') {
+			$indentBefore = $indent;
+		}
+		return c;
+	}
+EOF
+  fi
 }
 
 if [ -z "$LEXER" ]; then
-  _putreadf _read1 "$CTYPE" "$TYPE"
+  _putreadf _read1l "$CTYPE" "$TYPE"
+  cat << EOF
+
+	$CTYPE _read1() $tre {
+		$CTYPE c;
+
+		if(_unreadl != $nullsym) {
+			c = _unreadl;
+			_unreadl = $nullsym;
+EOF
+  if [ -z "$TYPE" -a -z "$NEWLINEMODE" ]; then
+    cat <<  EOF
+		} else if((c = _read1l()) == '\r' && (c = _read1l()) != '\n') {
+			_unreadl = c;
+			c = '\r';
+		}
+EOF
+  else
+    cat << EOF
+		} else {
+			c = _read1l();
+		}
+EOF
+  fi
+  cat << EOF
+		return c;
+	}
+EOF
 else
   _putreadf _read1l "$LCTYPE" "$LTYPE"
   nina_template.nfa.java.sub2.sh "$LEXER" "$LCTYPE" "$LTYPE" "$LEXER" "$LCTYPE"
   cat << EOF
 
-	$LCTYPE _read1ul($streamsym stream) $tre {
+	$LCTYPE _read1ul() $tre {
 		$LCTYPE c;
 
-		if(_unreadl == $nullsyml) {
-			c = _read1l(stream);
-		} else {
+		if(_unreadl != $nullsyml) {
 			c = _unreadl;
 			_unreadl = $nullsyml;
+EOF
+  if [ -z "$LTYPE" -a -z "$NEWLINEMODE" ]; then
+    cat <<  EOF
+		} else if((c = _read1l()) == '\r' && (c = _read1l()) != '\n') {
+			_unreadl = c;
+			c = '\r';
 		}
+EOF
+  else
+    cat << EOF
+		} else {
+			c = _read1l();
+		}
+EOF
+  fi
+  cat << EOF
 		return c;
 	}
 
-	$CTYPE _read1($streamsym stream) $tre {
+	$CTYPE _read1() $tre {
 		StringBuffer b = new StringBuffer();
 		Object o = null;
 		boolean f;
@@ -220,7 +429,7 @@ else
 		s = STATE;  STATE = 0;
 		f = ${LEXER}_accepted();
 		while(o == null) {
-			if((a = _read1ul(stream)) == $nullsyml) {
+			if((a = _read1ul()) == $nullsyml) {
 				o = ${LEXER}_gettoken(b);
 				break;
 			} else if(${LEXER}_step(a) == 0) {
@@ -255,7 +464,7 @@ fi
 
 cat << EOF
 
-	private $CTYPE _read($streamsym rd) $tre {
+	private $CTYPE _read() $tre {
 		$CTYPE c;
 
 		while(true) {
@@ -281,7 +490,7 @@ EOF
 				} else {
 					__lookahead = null;
 					__lookahead_ptr = -1;
-					c = _read(rd);
+					c = _read();
 				}
 				__logprint("Read Lookahead: ", c);
 EOF
@@ -292,11 +501,11 @@ EOF
 				} else {
 					__backtrack = null;
 					__backtrack_ptr = __backtrack_wptr = -1;
-					c = _read(rd);
+					c = _read();
 				}
 				__logprint("Read Backtracking: ", c);
 EOF
-echo "			} else if((c = _read1(rd)) != $nullsym) {"
+echo "			} else if((c = _read1()) != $nullsym) {"
 [ -n "$ENABLE_BACKTRACK" ] && echo '				__write_backtrack(c);'
 echo '				__logprint("Read: ", c);'
 [ -z "$READSLOW" ] && cat << EOF
@@ -447,15 +656,24 @@ then
 
 	private void __copy_lookahead(int p) {
 		$CTYPE[] a;
+		$CTYPE[] lookaheadOrg = __lookahead;
+		int lookaheadLength = __lookahead != null ? __lookahead.length : __lookahead_ptr;
 
-		if(__lookahead == null) {
-			a = new $CTYPE[__lookaheadw_ptr];
-		} else if(__lookaheadw_ptr < __lookahead.length) {
-			a = __lookahead;
-		} else {
-			a = new $CTYPE[__lookaheadw_ptr];
+		if(__lookahead_ptr > 0) {
+			a = new $CTYPE[__lookahead.length - __lookahead_ptr];
+			System.arraycopy(__lookahead, __lookahead_ptr, a, 0, a.length);
+			__lookahead = a;
 		}
+
+		a = new int[__lookaheadw_ptr + (lookaheadLength - __lookahead_ptr)];
 		System.arraycopy(__lookaheadw, 0, a, 0, __lookaheadw_ptr);
+		if(lookaheadOrg != null) {
+			System.arraycopy(lookaheadOrg,
+					__lookahead_ptr,
+					a,
+					__lookaheadw_ptr,
+					lookaheadLength - __lookahead_ptr);
+		}
 		__lookahead = a;
 		__lookahead_ptr = p;
 		__lookaheadw = null;
@@ -485,6 +703,10 @@ then
 		__lookahead_mark = __lookaheadw_ptr;
 	}
 
+	void LOOKAHEAD_MARK_INIT() {
+		__lookahead_mark = 0;
+	}
+
 EOF
 fi
 
@@ -597,7 +819,7 @@ for i in $SUBAUTOMATA
 do
   ch1=`replace_strange_char $i`
   echo
-  echo "	private int ${ch1}_step($streamsym __rd, $CTYPE" ' $c)' " $tre {"
+  echo "	private int ${ch1}_step($CTYPE" ' $c)' " $tre {"
   [ -n "$ENABLE_LOOKAHEAD" ] && echo '		boolean __l__ = __lookahead_ok;'
   [ -n "$ENABLE_LOOKAHEAD" ] && echo
   [ -n "$DYNAMICAUTOMATA" ]  && echo '		if((STATE & NINA_DISCARDSTATE) != 0)  return 1;'
@@ -634,8 +856,8 @@ do
 
 	private final Engine ENGINE_${ch1} = new Engine() {
 
-		int step($streamsym __rd, $CTYPE c) $tre {
-			return ${ch1}_step(__rd, c);
+		int step($CTYPE c) $tre {
+			return ${ch1}_step(c);
 		}
 
 		boolean accepted() {
@@ -674,6 +896,18 @@ EOF
     cat << EOF
 		}
 
+		boolean isDead() {
+EOF
+    print_deads $i
+    cat << EOF
+		}
+
+		boolean isEmptyTransition() {
+EOF
+    print_empty_transitions $i
+    cat << EOF
+		}
+
 		public String toString() {
 			return "${ch1}";
 		}
@@ -721,7 +955,7 @@ c='$c'
 			this(a.toString().toCharArray(), ost);
 		}
 
-		int step($streamsym __rd, $CTYPE $c) $tre {
+		int step($CTYPE $c) $tre {
 			if(ptr >= a.length) {
 				LOOKAHEAD_COMMIT();
 				return 0;
@@ -741,6 +975,14 @@ c='$c'
 			return true;
 		}
 
+		boolean isDead() {
+			return false;
+		}
+
+		boolean isEmptyTransition() {
+			return false;
+		}
+
 		int execaction($CTYPE c) {
 			return 1;
 		}
@@ -799,17 +1041,15 @@ EOF
 ch1=`replace_strange_char ${MAINNAME}`
 echo
 cat << EOF
-	private $CTYPE _parse($streamsym rd, $CTYPE x, Boolean rt,
-			boolean skip, int[] st) $tre {
+	private $CTYPE _parse($CTYPE x, Boolean rt, boolean skip,
+			int[] st) $tre {
 		boolean b = false, p = skip;
 		$CTYPE c = x;
 		Engine en;
 		int a;
 
 		b = __stk[__slen - 1].accepted();
-		if(rd == null) {
-			throw new RuntimeException("can not recurse");
-		} else if(rt.booleanValue()) {
+		if(rt.booleanValue()) {
 			switch(__stk[__slen - 1].execaction(NINA_BEGIN)) {
 			case NINA_ACCEPT:
 				__logprint("accept " + __stk[__slen - 1]);
@@ -835,7 +1075,7 @@ cat << EOF
 				en = __stk[__slen - 1];
 				if(p) {
 					p = false;
-				} else if((a = en.step(rd, c)) > 0) {
+				} else if((a = en.step(c)) > 0) {
 					__logprint("transit to state " + STATE + ": ", c);
 					b = en.accepted();
 					switch(en.execaction(c)) {
@@ -887,15 +1127,48 @@ EOF
 EOF
   cat << EOF
 				} else if(c == $nullsym) {
-					if(!b)  throw new ${TOKENERROR}();
-					st[0] = NINA_ACCEPT;  return $nullsym;
+EOF
+  [ -n "$MATCHER" ] || echo "					if(!b)  throw new ${TOKENERROR}();"
+  [ -n "$MATCHER" ] && cat << EOF
+					if(!b) {
+						st[0] = NINA_FAIL;
+						return $nullsym;
+					}
+EOF
+  cat << EOF
+					st[0] = NINA_ACCEPT;
+					return $nullsym;
 				} else {
 					__logprint("match failed: ", c);
 					__puttrace();
 					UNGET(c);
 					st[0] = NINA_FAIL;  return $nullsym;
 				}
-				c = _read(rd);
+
+				if(__stk[__slen - 1].isEmptyTransition()) {
+					// do nothing
+				} else if(!__stk[__slen - 1].isDead()) {
+					c = _read();
+				} else if(b) {
+					__logprint("accept " + __stk[__slen - 1]);
+					st[0] = NINA_ACCEPT;  return $nullsym;
+EOF
+  [ -n "$ENABLE_LOOKAHEAD" ] && cat << EOF
+				} else if(__lookaheadw_ptr >= 0) {
+					__logprint("match failed: try lookahead: ", c);
+					LOOKAHEAD_RB();
+					b = en.accepted();
+EOF
+  [ -n "$ENABLE_BACKTRACK" ] && cat << EOF
+				} else if(BACKTRACK(c)) {
+					__logprint("match failed: try backtracking: ", c);
+EOF
+  cat << EOF
+				} else {
+					__logprint("match failed: ", c);
+					__puttrace();
+					st[0] = NINA_FAIL;  return $nullsym;
+				}
 			} while(true);
 		} catch(RuntimeException e) {
 			UNGET(c);
@@ -929,7 +1202,7 @@ EOF
 		return __stk[__slen - 1].recover(e);
 	}
 
-	${API_ACCESS_MODIFIER} boolean parse($streamsym rd, Engine entry) $tre {
+	${API_ACCESS_MODIFIER}boolean parse(Engine entry) $tre {
 		Boolean b = Boolean.FALSE;
 		int[] a = new int[1];
 		boolean skip = true;
@@ -944,13 +1217,16 @@ EOF
 
 			ot: while(true) {
 				try {
-					if((c = _parse(rd, c, b, skip, a)) != $nullsym) {
+					if((c = _parse(c, b, skip, a)) != $nullsym) {
 						skip = false;
 					} else if(a[0] == NINA_FAIL) {
 						while((STATE = getdeadstate()) < 0) {
 							if((b = execfinally()) != null)  break ot;
 							if(__slen-- <= 1) {
-								throw new TokenException();
+EOF
+  [ -n "$MATCHER" ] || echo '								throw new TokenException();'
+  [ -n "$MATCHER" ] && echo '								return false;'
+  cat << EOF
 							}
 						}
 						skip = true;
@@ -983,38 +1259,42 @@ EOF
 				}
 				b = Boolean.TRUE;
 			}
-			if(!b.booleanValue())  throw new TokenException();
+EOF
+  [ -n "$MATCHER" ] || echo '			if(!b.booleanValue())  throw new TokenException();'
+  cat << EOF
 			return b.booleanValue();
 		} finally {
 			__logclose();
 		}
 	}
 
-	${API_ACCESS_MODIFIER} boolean parse($streamsym rd) $tre {
-		return parse(rd, ENGINE_${ch1});
+	${API_ACCESS_MODIFIER}boolean parse($streamsym rd) $tre {
+		streamStack.push(rd);
+		return parse(ENGINE_${ch1});
 	}
 
-	${API_ACCESS_MODIFIER} static boolean parseAll($streamsym rd) $tre {
+	${API_ACCESS_MODIFIER}static boolean parseAll($streamsym rd) $tre {
 		${CLASSNAME} o = new ${CLASSNAME}();
 
 		return o.parse(rd);
 	}
 
-	${API_ACCESS_MODIFIER} void setStream($streamsym rd) {
-		if(befstream != null) {
+	${API_ACCESS_MODIFIER}void setStream($streamsym rd) {
+		if(streamStack.size() == 0) {
 			throw new IllegalStateException();
 		}
-		yieldObject = befstream = rd;
+		yieldObject = rd;
+		streamStack.push(rd);
 	}
 
-	${API_ACCESS_MODIFIER} Object parseNext() $tre {
+	${API_ACCESS_MODIFIER}Object parseNext() $tre {
 		Object o;
 
-		if(befstream == null) {
+		if(streamStack.size() == 0) {
 			throw new IllegalStateException();
 		} else if(yieldObject == null) {
 			return null;
-		} else if(parse(befstream, ENGINE_${ch1})) {
+		} else if(parse(ENGINE_${ch1})) {
 			if(yieldObject == null)  throw new NullPointerException();
 			o = yieldObject;  yieldObject = null;
 			return o;
@@ -1024,111 +1304,18 @@ EOF
 		}
 	}
 
-EOF
-
-[ -z "$TYPE" ] && [ -n "$ENABLE_MATCHER" ] && cat << EOF
-	${API_ACCESS_MODIFIER} boolean matches(String s) {
-		int n;
-
-		try {
-			STATE = 0;  matched = null;
-			for(n = 0; n < s.length(); n++) {
-				if(${ch1}_step(null, s.charAt(n)) == 0) {
-					return false;
-				}
-			}
-		} catch(java.io.IOException e) {
-			throw new RuntimeException(e);
-		}
-
-		if(${ch1}_accepted()) {
-			matched = s;
-			return true;
-		} else {
-			return false;
-		}
-	}
-
-	${API_ACCESS_MODIFIER} boolean lookingAt(String s) {
-		StringBuffer b = new StringBuffer();
-		int n;
-
-		try {
-			STATE = 0;  matched = null;
-			for(n = 0; n < s.length(); n++) {
-				if(${ch1}_accepted()) {
-					matched = b.toString();
-				}
-
-				if(${ch1}_step(null, s.charAt(n)) > 0) {
-					// do nothing
-				} else {
-					return matched != null;
-				}
-				b.append(s.charAt(n));
-			}
-		} catch(java.io.IOException e) {
-			throw new RuntimeException(e);
-		}
-
-		if(${ch1}_accepted()) {
-			matched = b.toString();
-			return true;
-		} else {
-			return matched != null;
-		}
-	}
-
-	${API_ACCESS_MODIFIER} boolean find(String s) {
-		StringBuffer b;
-		int n, k;
-
-		try {
-			STATE = 0;  matched = null;
-			for(k = 0; k < s.length(); k++) {
-				b = new StringBuffer();
-				for(n = k; n < s.length(); n++) {
-					if(${ch1}_accepted()) {
-						matched = b.toString();
-					}
-
-					if(${ch1}_step(null, s.charAt(n)) > 0) {
-						// do noting
-					} else if(matched == null) {
-						break;
-					} else {
-						return true;
-					}
-					b.append(s.charAt(n));
-				}
-
-				if(${ch1}_accepted()) {
-					matched = b.toString();
-					return true;
-				} else if(matched != null) {
-					return true;
-				} else {
-					STATE = 0;
-				}
-			}
-			return false;
-		} catch(java.io.IOException e) {
-			throw new RuntimeException(e);
-		}
-	}
-
-	${API_ACCESS_MODIFIER} String group() {
-		return matched;
+	static void puts(String s) {
+		System.out.println(s);
 	}
 
 EOF
 
 [ -z "$TYPE" -o -z "$LTYPE" ] && [ -z "$NEWLINEMODE" ] && cat << EOF
-	${API_ACCESS_MODIFIER} boolean parse(java.io.InputStream rd) $tre {
+	${API_ACCESS_MODIFIER}boolean parse(java.io.InputStream rd) $tre {
 		return parse(new java.io.InputStreamReader(rd));
 	}
 
-	${API_ACCESS_MODIFIER} static boolean parseAll(
+	${API_ACCESS_MODIFIER}static boolean parseAll(
 			java.io.InputStream rd) $tre {
 		return parseAll(new java.io.InputStreamReader(rd));
 	}
@@ -1139,6 +1326,28 @@ for i in $SUBAUTOMATA
 do
   print_constants $i
 done
+echo '	/* @@@-PARSER-CODE-END-@@@ */'
+
 cat fragment
 #cat fragment | replace_action
+
+[ -n "$PUTMAIN" ] && cat << EOF
+	public static void main(String[] args) throws Exception {
+		parseAll(System.in);
+	}
+EOF
+[ -n "$PUTPROMPT" ] && cat << EOF
+	public static void main(String[] args) throws Exception {
+		String s;
+
+		while((s = System.console().readLine("> ")) != null) {
+			try {
+				parseAll(new java.io.StringReader(s));
+			} catch(TokenException e) {
+				System.out.println("syntax error");
+			}
+		}
+	}
+EOF
+
 echo '}'
